
% Script to analyse abrasion loss data using multiple regression

%
% Import data via File > Import Data... menu option
%
% Code assumes data read in as MATRIX
%
% Using menu options and selecting "Generate MATLAB script" option gives
% following code (I have just changed the filename in the xlsread function):

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Import the file
newData1 = importdata('abrasion.csv');

% Create new variables in the base workspace from those fields.
vars = fieldnames(newData1);
for i = 1:length(vars)
    assignin('base', vars{i}, newData1.(vars{i}));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% Produces matrix plot of data
%
figure;
plotmatrix(data)
title('Matrix plot of Abrasion Loss, Hardness and Tensile Strength','FontSize',16)
orient landscape
print -dpng abrasion.png

%
% Calculate correlation coefficinet
%
corr(data)

%
% Create design matrix X
%
y=data(:,1);
X=[ones(size(data,1),1) data(:,2:end)];

%
% Fit regression model (0.05 specifies 95% confidence level)
%
[B,BINT,R,RINT,STATS] = regress(y,X);

%
% Extract R-squared statistic
%
STATS(1)

% observation resampling (random X) - all three coefficients
nsim=10000;
bootbetas = bootstrp(nsim,@(y,x) regress(y,x),y,X);
prctile(bootbetas,[2.5 97.5],1)

figure;
subplot(1,3,1);hist(bootbetas(:,1),100);xlabel('Intercept','FontSize',16);ylabel('Frequency Counts','FontSize',16)
subplot(1,3,2);hist(bootbetas(:,2),100);xlabel('Hardness','FontSize',16)
subplot(1,3,3);hist(bootbetas(:,3),100);xlabel('Tensile Strength','FontSize',16)

% p-values for testing for null hypothesis of negative or positive coefficients
pnegative=sum(bootbetas>0)/nsim
ppositive=sum(bootbetas<0)/nsim

% Regression diagnostic plots
abrasionstats=regstats(y,data(:,2:end),'linear',{'beta','yhat','r','leverage','dffit'});

% Residuals against predictions
figure;
scatter(abrasionstats.yhat,abrasionstats.r,'filled')
hold on;
refline(0,0)
xlabel('Fitted Values','FontSize',16);ylabel('Residuals','FontSize',16);
       
% Residuals against explanatory variables
figure;
scatter(data(:,2),abrasionstats.r,'filled')
hold on;
refline(0,0)
xlabel('Hardness','FontSize',24);ylabel('Residuals','FontSize',24);
       
figure;
scatter(data(:,3),abrasionstats.r,'filled')
hold on;
refline(0,0)
xlabel('Tensile Strength','FontSize',24);ylabel('Residuals','FontSize',24);
       
figure;
scatter(abrasionstats.leverage,abrasionstats.dffit,'filled')
hold on;
refline(0,0)
xlabel('Leverages','FontSize',16);ylabel('Leave-one-out Change in Predictions','FontSize',16);

% Find high leverage outlier (which row)
find(abrasionstats.dffit< -10)

% Highlight this datapoint on scatterplot
figure;
scatter(data(:,3),data(:,1),'filled')
xlabel('Tensile Strength','FontSize',24);ylabel('Abrasion Loss','FontSize',24);
hold on;
plot(data(19,3),data(19,1),'x','MarkerSize',20)

%
% Repeat bootstrap ignoring this datapoint and compare coefficients for
% Tensile strength
%
% observation resampling (random X)
bootbetas19 = bootstrp(nsim,@(y,x) regress(y,x),y([1:18,20:end]),X([1:18,20:end],:));
prctile(bootbetas19,[2.5 97.5],1)

figure;
subplot(2,1,1);hist(bootbetas(:,3),100);xlabel('Tensile Strength','FontSize',16);axis([-2 -0.8 0 450]);title('Including All Data','FontSize',16)
subplot(2,1,2);hist(bootbetas19(:,3),100);xlabel('Tensile Strength','FontSize',16);axis([-2 -0.8 0 450]);title('Ignoring Outlier','FontSize',16)

% 10-fold Cross-validation of all 3 models of interest

regf = @(Xtrain, ytrain, Xtest)(Xtest * regress(ytrain,Xtrain))
cvMse=crossval('mse',X,y,'predfun',regf)
cvMseNoStrength=crossval('mse',X(:,1:2),y,'predfun',regf)
cvMseHardness=crossval('mse',X(:,[1,3]),y,'predfun',regf)
 
[cvMse cvMseNoStrength cvMseHardness]
 
% Adjusted R-squared for all 3 models of interest
regstats(y,data(:,2:end),'linear',{'adjrsquare'})
regstats(y,data(:,2),'linear',{'adjrsquare'})
regstats(y,data(:,3),'linear',{'adjrsquare'})



